@@ -280,7 +280,8 @@ do_lookup_password (GckSession *session, const gchar *keyid)
 	for (l = objects; l; l = g_list_next (l)) {
 		data = gck_object_get_data_full (l->data, CKA_VALUE, egg_secure_realloc, NULL, &n_data, &error);
 		if (error) {
-			g_warning ("couldn't lookup gpg agent password: %s", egg_error_message (error));
+			if (!g_error_matches (error, GCK_ERROR, CKR_USER_NOT_LOGGED_IN))
+				g_warning ("couldn't lookup gpg agent password: %s", egg_error_message (error));
 			g_clear_error (&error);
 			data = NULL;
 		} else {
@@ -631,11 +632,12 @@ command_has_option (gchar *command, gchar *option)
 	return has_option;
 }
 
+static const char HEXC[] = "0123456789abcdef";
+
 /* Encode a password in hex */
 static gchar*
-encode_password (const gchar *pass)
+hex_encode_password (const gchar *pass)
 {
-	static const char HEXC[] = "0123456789abcdef";
 	int j, c;
 	gchar *enc, *k;
 
@@ -655,6 +657,36 @@ encode_password (const gchar *pass)
 	return enc;
 }
 
+static gchar*
+uri_encode_password (const gchar *value)
+{
+	gchar *p;
+	gchar *result;
+
+	/* Just allocate for worst case */
+	result = egg_secure_alloc ((strlen (value) * 3) + 1);
+
+	/* Now loop through looking for escapes */
+	p = result;
+	while (*value) {
+
+		/* These characters we let through verbatim */
+		if (*value && (g_ascii_isalnum (*value) || strchr ("_-.", *value) != NULL)) {
+			*(p++) = *(value++);
+
+		/* All others get encoded */
+		} else {
+			*(p++) = '%';
+			*(p++) = HEXC[((unsigned char)*value) >> 4];
+			*(p++) = HEXC[((unsigned char)*value) & 0x0F];
+			++value;
+		}
+	}
+
+	*p = 0;
+	return result;
+}
+
 /* ----------------------------------------------------------------------------------
  * OPERATIONS
  */
@@ -736,10 +768,12 @@ gkd_gpg_agent_ops_getpass (GkdGpgAgentCall *call, gchar *args)
 	if (password == NULL) {
 		gkd_gpg_agent_send_reply (call, FALSE, "111 cancelled");
 	} else if (flags & GKD_GPG_AGENT_PASS_AS_DATA) {
-		gkd_gpg_agent_send_data (call, password);
+		encoded = uri_encode_password (password);
+		gkd_gpg_agent_send_data (call, encoded);
 		gkd_gpg_agent_send_reply (call, TRUE, NULL);
+		egg_secure_strfree (encoded);
 	} else {
-		encoded = encode_password (password);
+		encoded = hex_encode_password (password);
 		gkd_gpg_agent_send_reply (call, TRUE, encoded);
 		egg_secure_strfree (encoded);
 	}
